@@ -28,6 +28,7 @@
 #include "CommandDialog.hh"
 #include "fluxbox.hh"
 
+#include "FbTk/MemFun.hh"
 #include "FbTk/SimpleCommand.hh"
 #include "FbTk/ImageControl.hh"
 #include "FbTk/TextUtils.hh"
@@ -168,7 +169,7 @@ ClockTool::ClockTool(const FbTk::FbWindow &parent,
                  screen.name() + ".strftimeFormat", screen.altName() + ".StrftimeFormat"),
     m_stringconvertor(FbTk::StringConvertor::ToFbString) {
     // attach signals
-    theme.reconfigSig().attach(this);
+    m_tracker.join(theme.reconfigSig(), FbTk::MemFun(*this, &ClockTool::themeReconfigured));
 
     std::string time_locale = setlocale(LC_TIME, NULL);
     size_t pos = time_locale.find('.');
@@ -197,7 +198,7 @@ ClockTool::ClockTool(const FbTk::FbWindow &parent,
     menu.insert(_FB_XTEXT(Toolbar, ClockEditFormat,   "Edit Clock Format",   "edit Clock Format") , editformat_cmd);
 
 
-    update(0);
+    themeReconfigured();
 }
 
 ClockTool::~ClockTool() {
@@ -233,10 +234,10 @@ void ClockTool::hide() {
 
 void ClockTool::setTimeFormat(const std::string &format) {
     *m_timeformat = format;
-    update(0);
+    themeReconfigured();
 }
 
-void ClockTool::update(FbTk::Subject *subj) {
+void ClockTool::themeReconfigured() {
     updateTime();
 
     // + 2 to make the entire text fit inside
@@ -259,7 +260,7 @@ void ClockTool::update(FbTk::Subject *subj) {
     translateSize(orientation(), new_width, new_height);
     if (new_width != m_button.width() || new_height != m_button.height()) {
         resize(new_width, new_height);
-        resizeSig().notify();
+        resizeSig().emit();
     }
 
 }
@@ -304,7 +305,7 @@ void ClockTool::updateTime() {
         unsigned int new_width = m_theme->font().textWidth(time_string, time_string_len) + 2;
         if (new_width > m_button.width()) {
             resize(new_width, m_button.height());
-            resizeSig().notify();
+            resizeSig().emit();
         }
 #else // dont have strftime so we have to set it to hour:minut
         //        sprintf(time_string, "%d:%d", );
@@ -316,7 +317,7 @@ void ClockTool::updateTime() {
 void ClockTool::updateSizing() {
     m_button.setBorderWidth(m_theme->border().width());
     // resizes if new timeformat
-    update(0);
+    themeReconfigured();
 }
 
 void ClockTool::reRender() {
