@@ -100,7 +100,7 @@ void resetEffects(FbTk::Font& font) {
     font.setShadowOffX(2);
 }
 
-}; // end nameless namespace
+} // end nameless namespace
 
 
 
@@ -247,7 +247,7 @@ bool Font::load(const string &name) {
     return false;
 }
 
-unsigned int Font::textWidth(const FbString &text, unsigned int size) const {
+unsigned int Font::textWidth(const char* text, unsigned int size) const {
     return m_fontimp->textWidth(text, size);
 }
 
@@ -268,38 +268,31 @@ bool Font::validOrientation(FbTk::Orientation orient) {
 }
 
 void Font::drawText(const FbDrawable &w, int screen, GC gc,
-                    const FbString &text, size_t len, int x, int y,
+                    const char* text, size_t len, int x, int y,
                     Orientation orient) const {
-    if (text.empty() || len == 0)
+
+    if (!text || !*text || len == 0)
         return;
 
-    // so we don't end up in a loop with m_shadow
-    static bool first_run = true;
-
     // draw "effects" first
-    if (first_run) {
-        if (m_shadow) {
-            FbTk::GContext shadow_gc(w);
-            shadow_gc.setForeground(m_shadow_color);
-            first_run = false;
-            drawText(w, screen, shadow_gc.gc(), text, len,
-                     x + m_shadow_offx, y + m_shadow_offy, orient);
-            first_run = true;
-        } else if (m_halo) {
-            FbTk::GContext halo_gc(w);
-            halo_gc.setForeground(m_halo_color);
-            first_run = false;
-            drawText(w, screen, halo_gc.gc(), text, len, x + 1, y + 1, orient);
-            drawText(w, screen, halo_gc.gc(), text, len, x - 1, y + 1, orient);
-            drawText(w, screen, halo_gc.gc(), text, len, x - 1, y - 1, orient);
-            drawText(w, screen, halo_gc.gc(), text, len, x + 1, y - 1, orient);
-            first_run = true;
-        }
+    if (m_shadow) {
+        FbTk::GContext shadow_gc(w);
+        shadow_gc.setForeground(m_shadow_color);
+        m_fontimp->drawText(w, screen, shadow_gc.gc(), text, len,
+                 x + m_shadow_offx, y + m_shadow_offy, orient);
+    } else if (m_halo) {
+        FbTk::GContext halo_gc(w);
+        halo_gc.setForeground(m_halo_color);
+        m_fontimp->drawText(w, screen, halo_gc.gc(), text, len, x + 1, y + 1, orient);
+        m_fontimp->drawText(w, screen, halo_gc.gc(), text, len, x - 1, y + 1, orient);
+        m_fontimp->drawText(w, screen, halo_gc.gc(), text, len, x - 1, y - 1, orient);
+        m_fontimp->drawText(w, screen, halo_gc.gc(), text, len, x + 1, y - 1, orient);
     }
 
     m_fontimp->drawText(w, screen, gc, text, len, x, y, orient);
 
+
 }
 
-};
+}
 
