@@ -22,11 +22,10 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: ImageControl.cc 4199 2006-02-16 06:53:05Z mathias $
-
 #include "ImageControl.hh"
 
 #include "TextureRender.hh"
+#include "Texture.hh"
 #include "App.hh"
 #include "SimpleCommand.hh"
 #include "I18n.hh"
@@ -50,65 +49,139 @@
 #else
   #include <stdlib.h>
 #endif
-#ifdef HAVE_CSTRING
-  #include <cstring>
-#else
-  #include <string.h>
-#endif
 #ifdef HAVE_CSTDIO
   #include <cstdio>
 #else
   #include <stdio.h>
 #endif
 
-#ifdef HAVE_CTYPE_H
-#include <ctype.h>
-#endif // HAVE_CTYPE_H
-
 #include <iostream>
 
-using namespace std;
+using std::cerr;
+using std::endl;
+using std::list;
 
 namespace FbTk {
 
-// lookup table for texture
-unsigned long *ImageControl::sqrt_table = 0;
-#ifdef TIMEDCACHE
-bool ImageControl::s_timed_cache = true;
-#else
-bool ImageControl::s_timed_cache = false;
-#endif // TIMEDCACHE
-
 namespace { // anonymous
 
-inline unsigned long bsqrt(unsigned long x) {
-    if (x <= 0) return 0;
-    if (x == 1) return 1;
+#ifdef TIMEDCACHE
+bool s_timed_cache = true;
+#else
+bool s_timed_cache = false;
+#endif // TIMEDCACHE
 
-    unsigned long r = x >> 1;
-    unsigned long q;
 
-    while (1) {
-        q = x / r;
-        if (q >= r) return r;
-        r = (r + q) >> 1;
+void initColortables(unsigned char red[256], unsigned char green[256], unsigned char blue[256],
+      int red_bits, int green_bits, int blue_bits) {
+
+    for (unsigned int i = 0; i < 256; i++) {
+        red[i] = i / red_bits;
+        green[i] = i / green_bits;
+        blue[i] = i / blue_bits;
     }
 }
 
-}; // end anonymous namespace
+// tries to allocate all unallocated 'colors' by finding a close color based
+// upon entries in the colormap.
+//
+void allocateUnallocatedColors(std::vector<XColor> colors, Display* dpy, Colormap cmap, int screen_depth) {
 
-ImageControl::ImageControl(int screen_num, bool dither,
+    unsigned int i;
+
+    bool done = true;
+
+    // first run, just try to allocate the colors
+    for (i = 0; i < colors.size(); i++) {
+
+        if (colors[i].flags == 0) {
+            if (! XAllocColor(dpy, cmap, &colors[i])) {
+                fprintf(stderr, "couldn't alloc color %i %i %i\n",
+                        colors[i].red, colors[i].green, colors[i].blue);
+                colors[i].flags = 0;
+                done = false;
+            } else
+                colors[i].flags = DoRed|DoGreen|DoBlue;
+        }
+    }
+
+    if (done)
+        return;
+
+    // 'icolors' will hold the first 'nr_icolors' colors of the
+    // given (indexed) colormap.
+    const size_t nr_icolors = std::min(256, 1 << screen_depth);
+    std::vector<XColor> icolors(nr_icolors);
+
+    // give each icolor an index
+    for (i = 0; i < nr_icolors; i++)
+        icolors[i].pixel = i;
+
+    // query the colors of the colormap and store them into 'icolors'
+    XQueryColors(dpy, cmap, &icolors[0], nr_icolors);
+
+    // try to find a close color for all not allocated colors
+    for (i = 0; i < colors.size(); i++) {
+
+        if (colors[i].flags == 0) { // color is not allocated
+
+            unsigned long chk = 0xffffffff;
+            unsigned long close = 0;
+
+            // iterate over the indexed colors 'icolors' and find
+            // a close color. 
+            //
+            // 2 passes to improve the result of the first pass
+
+            char pass = 2;
+            while (pass--) {
+                for (unsigned int ii = 0; ii < nr_icolors; ii++) {
+
+                    int r = (colors[i].red - icolors[i].red) >> 8;
+                    int g = (colors[i].green - icolors[i].green) >> 8;
+                    int b = (colors[i].blue - icolors[i].blue) >> 8;
+                    unsigned long pixel = (r * r) + (g * g) + (b * b);
+
+                    if (pixel < chk) {
+                        chk = pixel;
+                        close = ii;
+                    }
+
+                    // store the indexed color
+                    colors[i].red = icolors[close].red;
+                    colors[i].green = icolors[close].green;
+                    colors[i].blue = icolors[close].blue;
+
+                    // try to allocate it
+                    if (XAllocColor(dpy, cmap, &colors[i])) {
+                        colors[i].flags = DoRed|DoGreen|DoBlue; // mark it allocated
+                        break;
+                    }
+                }
+            }
+        }
+    }
+}
+
+
+} // end anonymous namespace
+
+struct ImageControl::Cache {
+    Pixmap pixmap;
+    Pixmap texture_pixmap;
+    Orientation orient;
+    unsigned int count, width, height;
+    unsigned long pixel1, pixel2, texture;
+};
+
+ImageControl::ImageControl(int screen_num,
                            int cpc, unsigned long cache_timeout, unsigned long cmax):
-    m_dither(dither),
-    m_colors(0),
-    m_num_colors(0),
-    m_colors_per_channel(cpc) {
+    m_colors_per_channel(cpc),
+    m_screen_num(screen_num) {
 
     Display *disp = FbTk::App::instance()->display();
 
     m_screen_depth = DefaultDepth(disp, screen_num);
-    m_screen_num = screen_num;
-    m_root_window = RootWindow(disp, screen_num);
     m_visual = DefaultVisual(disp, screen_num);
     m_colormap = DefaultColormap(disp, screen_num);
 
@@ -116,7 +189,7 @@ ImageControl::ImageControl(int screen_num, bool dither,
 
     if (cache_timeout && s_timed_cache) {
         m_timer.setTimeout(cache_timeout);
-        RefCount<Command> clean_cache(new SimpleCommand<ImageControl>(*this, &ImageControl::cleanCache));
+        RefCount<Command<void> > clean_cache(new SimpleCommand<ImageControl>(*this, &ImageControl::cleanCache));
         m_timer.setCommand(clean_cache);
         m_timer.start();
     }
@@ -126,47 +199,31 @@ ImageControl::ImageControl(int screen_num, bool dither,
 
 
 ImageControl::~ImageControl() {
-    if (sqrt_table) {
-        delete [] sqrt_table;
-        sqrt_table = 0;
-    }
-
-    if (grad_xbuffer) {
-        delete [] grad_xbuffer;
-    }
-
-    if (grad_ybuffer) {
-        delete [] grad_ybuffer;
-    }
 
     Display *disp = FbTk::App::instance()->display();
 
-    if (m_colors) {
-        unsigned long *pixels = new unsigned long [m_num_colors];
+    if (!m_colors.empty()) {
+        std::vector<unsigned long> pixels(m_colors.size());
 
-        for (unsigned int color = 0; color < m_num_colors; color++)
-            *(pixels + color) = (*(m_colors + color)).pixel;
+        for (unsigned int i = 0; i < m_colors.size(); i++)
+            pixels[i] = m_colors[i].pixel;
 
-        XFreeColors(disp, m_colormap, pixels, m_num_colors, 0);
-
-        delete [] m_colors;
+        XFreeColors(disp, m_colormap, &pixels[0], pixels.size(), 0);
     }
 
-    if (cache.size() > 0) {
+    if (!cache.empty()) {
         CacheList::iterator it = cache.begin();
         CacheList::iterator it_end = cache.end();
         for (; it != it_end; ++it) {
             XFreePixmap(disp, (*it)->pixmap);
             delete (*it);
         }
-
     }
-
 }
 
 
 Pixmap ImageControl::searchCache(unsigned int width, unsigned int height,
-                                 const Texture &text) const {
+                                 const Texture &text, FbTk::Orientation orient) const {
 
     if (text.pixmap().drawable() != None) {
         // do comparsion with width/height and texture_pixmap
@@ -174,7 +231,9 @@ Pixmap ImageControl::searchCache(unsigned int width, unsigned int height,
         CacheList::iterator it_end = cache.end();
         for (; it != it_end; ++it) {
             if ((*it)->texture_pixmap == text.pixmap().drawable() &&
-                (*it)->width == width && (*it)->height == height &&
+                (*it)->orient == orient &&
+                (*it)->width == width &&
+                (*it)->height == height &&
                 (*it)->texture == text.type()) {
                 (*it)->count++;
                 return (*it)->pixmap;
@@ -196,6 +255,7 @@ Pixmap ImageControl::searchCache(unsigned int width, unsigned int height,
     CacheList::iterator it_end = cache.end();
     for (; it != it_end; ++it) {
         if (((*it)->width == width) &&
+            ((*it)->orient == orient) &&
             ((*it)->height == height) &&
             ((*it)->texture == text.type()) &&
             ((*it)->pixel1 == text.color().pixel())) {
@@ -217,19 +277,28 @@ Pixmap ImageControl::searchCache(unsigned int width, unsigned int height,
 
 
 Pixmap ImageControl::renderImage(unsigned int width, unsigned int height,
-                                 const FbTk::Texture &texture) {
+                                 const FbTk::Texture &texture,
+                                 FbTk::Orientation orient,
+                                 bool use_cache ) {
 
     if (texture.type() & FbTk::Texture::PARENTRELATIVE)
         return ParentRelative;
 
+    // If we are not suppose to cache this pixmap, just render and return it
+    if ( ! use_cache) {
+        TextureRender image(*this, width, height, orient);
+        return image.render(texture);
+    }
+
     // search cache first
-    Pixmap pixmap = searchCache(width, height, texture);
+    Pixmap pixmap = searchCache(width, height, texture, orient);
     if (pixmap) {
         return pixmap; // return cache item
     }
 
     // render new image
-    TextureRender image(*this, width, height, m_colors, m_num_colors);
+
+    TextureRender image(*this, width, height, orient);
     pixmap = image.render(texture);
 
     if (pixmap) {
@@ -239,6 +308,7 @@ Pixmap ImageControl::renderImage(unsigned int width, unsigned int height,
 
         tmp->pixmap = pixmap;
         tmp->texture_pixmap = texture.pixmap().drawable();
+        tmp->orient = orient;
         tmp->width = width;
         tmp->height = height;
         tmp->count = 1;
@@ -252,7 +322,7 @@ Pixmap ImageControl::renderImage(unsigned int width, unsigned int height,
 
         cache.push_back(tmp);
 
-        if ((unsigned) cache.size() > cache_max)
+        if (cache.size() > cache_max)
             cleanCache();
 
         return pixmap;
@@ -305,40 +375,19 @@ void ImageControl::colorTables(const unsigned char **rmt, const unsigned char **
     if (bbit) *bbit = blue_bits;
 }
 
-
-void ImageControl::getXColorTable(XColor **c, int *n) {
-    if (c) *c = m_colors;
-    if (n) *n = m_num_colors;
-}
-
-
 void ImageControl::getGradientBuffers(unsigned int w,
                                       unsigned int h,
                                       unsigned int **xbuf,
                                       unsigned int **ybuf) {
 
-    if (w > grad_buffer_width) {
-        if (grad_xbuffer) {
-            delete [] grad_xbuffer;
-        }
+    if (w > grad_xbuffer.size())
+        grad_xbuffer.resize(w);
 
-        grad_buffer_width = w;
+    if (h > grad_ybuffer.size())
+        grad_ybuffer.resize(h);
 
-        grad_xbuffer = new unsigned int[grad_buffer_width * 3];
-    }
-
-    if (h > grad_buffer_height) {
-        if (grad_ybuffer) {
-            delete [] grad_ybuffer;
-        }
-
-        grad_buffer_height = h;
-
-        grad_ybuffer = new unsigned int[grad_buffer_height * 3];
-    }
-
-    *xbuf = grad_xbuffer;
-    *ybuf = grad_ybuffer;
+    *xbuf = &grad_xbuffer[0];
+    *ybuf = &grad_ybuffer[0];
 }
 
 
@@ -350,7 +399,7 @@ void ImageControl::installRootColormap() {
     bool install = true;
     int i = 0, ncmap = 0;
     Colormap *cmaps =
-        XListInstalledColormaps(disp, m_root_window, &ncmap);
+        XListInstalledColormaps(disp, RootWindow(disp, screenNumber()), &ncmap);
 
     if (cmaps) {
         for (i = 0; i < ncmap; i++) {
@@ -368,31 +417,10 @@ void ImageControl::installRootColormap() {
 }
 
 
-void ImageControl::setColorsPerChannel(int cpc) {
-    if (cpc < 2) cpc = 2;
-    if (cpc > 6) cpc = 6;
-
-    m_colors_per_channel = cpc;
-}
-
-
-unsigned long ImageControl::getSqrt(unsigned int x) const {
-    if (! sqrt_table) {
-        // build sqrt table for use with elliptic gradient
-
-        sqrt_table = new unsigned long[256 * 256 * 2 + 1];
-        int i = 0;
-
-        for (; i < (256 * 256 * 2); i++)
-            sqrt_table[i] = bsqrt(i);
-    }
-
-    return sqrt_table[x];
-}
 
 void ImageControl::cleanCache() {
     Display *disp = FbTk::App::instance()->display();
-    std::list<CacheList::iterator> deadlist;
+    list<CacheList::iterator> deadlist;
     CacheList::iterator it = cache.begin();
     CacheList::iterator it_end = cache.end();
     for (; it != it_end; ++it) {
@@ -405,8 +433,8 @@ void ImageControl::cleanCache() {
         }
     }
 
-    std::list<CacheList::iterator>::iterator dead_it = deadlist.begin();
-    std::list<CacheList::iterator>::iterator dead_it_end = deadlist.end();
+    list<CacheList::iterator>::iterator dead_it = deadlist.begin();
+    list<CacheList::iterator>::iterator dead_it_end = deadlist.end();
     for (; dead_it != dead_it_end; ++dead_it) {
         cache.erase(*dead_it);
     }
@@ -416,9 +444,6 @@ void ImageControl::cleanCache() {
 void ImageControl::createColorTable() {
     Display *disp = FbTk::App::instance()->display();
 
-    grad_xbuffer = grad_ybuffer = (unsigned int *) 0;
-    grad_buffer_width = grad_buffer_height = 0;
-
     int count;
     XPixmapFormatValues *pmv = XListPixmapFormats(disp, &count);
 
@@ -436,16 +461,11 @@ void ImageControl::createColorTable() {
 
     if (bits_per_pixel == 0)
         bits_per_pixel = m_screen_depth;
-    if (bits_per_pixel >= 24)
-        setDither(false);
 
     red_offset = green_offset = blue_offset = 0;
 
     switch (visual()->c_class) {
     case TrueColor: {
-        int i;
-
-        // compute color tables
         unsigned long red_mask = visual()->red_mask,
             green_mask = visual()->green_mask,
             blue_mask = visual()->blue_mask;
@@ -458,11 +478,8 @@ void ImageControl::createColorTable() {
         green_bits = 255 / green_mask;
         blue_bits = 255 / blue_mask;
 
-        for (i = 0; i < 256; i++) {
-            red_color_table[i] = i / red_bits;
-            green_color_table[i] = i / green_bits;
-            blue_color_table[i] = i / blue_bits;
-        }
+        initColortables(red_color_table, green_color_table, blue_color_table,
+                red_bits, green_bits, blue_bits);
     }
 
         break;
@@ -470,193 +487,94 @@ void ImageControl::createColorTable() {
     case PseudoColor:
     case StaticColor: {
 
-        m_num_colors = m_colors_per_channel * m_colors_per_channel * m_colors_per_channel;
+        size_t num_colors = m_colors_per_channel * m_colors_per_channel * m_colors_per_channel;
 
-        if (m_num_colors > static_cast<unsigned int>(1 << m_screen_depth)) {
+        if (num_colors > static_cast<unsigned>(1 << m_screen_depth)) {
             m_colors_per_channel = (1 << m_screen_depth) / 3;
-            m_num_colors = m_colors_per_channel * m_colors_per_channel * m_colors_per_channel;
+            num_colors = m_colors_per_channel * m_colors_per_channel * m_colors_per_channel;
         }
 
-        if (m_colors_per_channel < 2 || m_num_colors > static_cast<unsigned int>(1 << m_screen_depth)) {
-            fprintf(stderr, "ImageControl::ImageControl: invalid colormap size %d "
+        if (m_colors_per_channel < 2 || num_colors > static_cast<unsigned>(1 << m_screen_depth)) {
+            fprintf(stderr, "ImageControl::ImageControl: invalid colormap size %ld "
                     "(%d/%d/%d) - reducing",
-                    m_num_colors, m_colors_per_channel, m_colors_per_channel,
+                    num_colors, m_colors_per_channel, m_colors_per_channel,
                     m_colors_per_channel);
 
             m_colors_per_channel = (1 << m_screen_depth) / 3;
         }
 
-        m_colors = new XColor[m_num_colors];
-
-        int bits = 256 / m_colors_per_channel;
-
-#ifndef ORDEREDPSEUDO
-        bits = 255 / (m_colors_per_channel - 1);
-#endif // ORDEREDPSEUDO
+        m_colors.resize(num_colors);
 
+        int bits = 255 / (m_colors_per_channel - 1);
         red_bits = green_bits = blue_bits = bits;
 
-        for (unsigned int i = 0; i < 256; i++) {
-            red_color_table[i] = green_color_table[i] = blue_color_table[i] =
-                i / bits;
-        }
+        initColortables(red_color_table, green_color_table, blue_color_table,
+                red_bits, green_bits, blue_bits);
 
         for (int r = 0, i = 0; r < m_colors_per_channel; r++) {
             for (int g = 0; g < m_colors_per_channel; g++) {
                 for (int b = 0; b < m_colors_per_channel; b++, i++) {
                     m_colors[i].red = (r * 0xffff) / (m_colors_per_channel - 1);
                     m_colors[i].green = (g * 0xffff) / (m_colors_per_channel - 1);
-                    m_colors[i].blue = (b * 0xffff) / (m_colors_per_channel - 1);;
+                    m_colors[i].blue = (b * 0xffff) / (m_colors_per_channel - 1);
                     m_colors[i].flags = DoRed|DoGreen|DoBlue;
                 }
             }
         }
 
-        for (unsigned int i = 0; i < m_num_colors; i++) {
-            if (! XAllocColor(disp, m_colormap, &m_colors[i])) {
-                fprintf(stderr, "couldn't alloc color %i %i %i\n",
-                        m_colors[i].red, m_colors[i].green, m_colors[i].blue);
-                m_colors[i].flags = 0;
-            } else
-                m_colors[i].flags = DoRed|DoGreen|DoBlue;
-        }
-
-        XColor icolors[256];
-        unsigned int incolors = (((1 << m_screen_depth) > 256) ? 256 : (1 << m_screen_depth));
-
-        for (unsigned int i = 0; i < incolors; i++)
-            icolors[i].pixel = i;
-
-        XQueryColors(disp, m_colormap, icolors, incolors);
-        for (unsigned int i = 0; i < m_num_colors; i++) {
-            if (! m_colors[i].flags) {
-                unsigned long chk = 0xffffffff, pixel, close = 0;
-                char p = 2;
-
-                while (p--) {
-                    for (unsigned int ii = 0; ii < incolors; ii++) {
-                        int r = (m_colors[i].red - icolors[i].red) >> 8;
-                        int g = (m_colors[i].green - icolors[i].green) >> 8;
-                        int b = (m_colors[i].blue - icolors[i].blue) >> 8;
-                        pixel = (r * r) + (g * g) + (b * b);
-
-                        if (pixel < chk) {
-                            chk = pixel;
-                            close = ii;
-                        }
-
-                        m_colors[i].red = icolors[close].red;
-                        m_colors[i].green = icolors[close].green;
-                        m_colors[i].blue = icolors[close].blue;
-
-                        if (XAllocColor(disp, m_colormap,
-                                        &m_colors[i])) {
-                            m_colors[i].flags = DoRed|DoGreen|DoBlue;
-                            break;
-                        }
-                    }
-                }
-            }
-        }
-
+        allocateUnallocatedColors(m_colors, disp, m_colormap, m_screen_depth);
         break;
     }
 
     case GrayScale:
     case StaticGray:
         {
+            size_t num_colors;
 
             if (visual()->c_class == StaticGray) {
-                m_num_colors = 1 << m_screen_depth;
+                num_colors = 1 << m_screen_depth;
             } else {
-                m_num_colors = m_colors_per_channel * m_colors_per_channel * m_colors_per_channel;
+                num_colors = m_colors_per_channel * m_colors_per_channel * m_colors_per_channel;
 
-                if (m_num_colors > static_cast<unsigned int>(1 << m_screen_depth)) {
+                if (num_colors > static_cast<unsigned>(1 << m_screen_depth)) {
                     m_colors_per_channel = (1 << m_screen_depth) / 3;
-                    m_num_colors = m_colors_per_channel * m_colors_per_channel * m_colors_per_channel;
+                    num_colors = m_colors_per_channel * m_colors_per_channel * m_colors_per_channel;
                 }
             }
 
-            if (m_colors_per_channel < 2 || m_num_colors > static_cast<unsigned int>(1 << m_screen_depth)) {
-                fprintf(stderr,"FbTk::ImageControl: invalid colormap size %d "
+            if (m_colors_per_channel < 2 || num_colors > static_cast<unsigned>(1 << m_screen_depth)) {
+                fprintf(stderr,"FbTk::ImageControl: invalid colormap size %ld "
                         "(%d/%d/%d) - reducing",
-                        m_num_colors, m_colors_per_channel, m_colors_per_channel,
+                        num_colors, m_colors_per_channel, m_colors_per_channel,
                         m_colors_per_channel);
 
                 m_colors_per_channel = (1 << m_screen_depth) / 3;
             }
 
-            m_colors = new XColor[m_num_colors];
+            m_colors.resize(num_colors);
 
-            int p, bits = 255 / (m_colors_per_channel - 1);
+            int bits = 255 / (m_colors_per_channel - 1);
             red_bits = green_bits = blue_bits = bits;
 
-            for (unsigned int i = 0; i < 256; i++)
-                red_color_table[i] = green_color_table[i] = blue_color_table[i] =
-                    i / bits;
+            initColortables(red_color_table, green_color_table, blue_color_table,
+                red_bits, green_bits, blue_bits);
 
-            for (unsigned int i = 0; i < m_num_colors; i++) {
+            for (unsigned int i = 0; i < num_colors; i++) {
                 m_colors[i].red = (i * 0xffff) / (m_colors_per_channel - 1);
                 m_colors[i].green = (i * 0xffff) / (m_colors_per_channel - 1);
-                m_colors[i].blue = (i * 0xffff) / (m_colors_per_channel - 1);;
+                m_colors[i].blue = (i * 0xffff) / (m_colors_per_channel - 1);
                 m_colors[i].flags = DoRed|DoGreen|DoBlue;
-
-                if (! XAllocColor(disp, m_colormap,
-                                  &m_colors[i])) {
-                    fprintf(stderr, "Couldn't alloc color %i %i %i\n",
-                            m_colors[i].red, m_colors[i].green, m_colors[i].blue);
-                    m_colors[i].flags = 0;
-                } else
-                    m_colors[i].flags = DoRed|DoGreen|DoBlue;
-            }
-
-
-            XColor icolors[256];
-            unsigned int incolors = (((1 << m_screen_depth) > 256) ? 256 :
-                                     (1 << m_screen_depth));
-
-            for (unsigned int i = 0; i < incolors; i++)
-                icolors[i].pixel = i;
-
-            XQueryColors(disp, m_colormap, icolors, incolors);
-            for (unsigned int i = 0; i < m_num_colors; i++) {
-                if (! m_colors[i].flags) {
-                    unsigned long chk = 0xffffffff, pixel, close = 0;
-
-                    p = 2;
-                    while (p--) {
-                        for (unsigned int ii = 0; ii < incolors; ii++) {
-                            int r = (m_colors[i].red - icolors[i].red) >> 8;
-                            int g = (m_colors[i].green - icolors[i].green) >> 8;
-                            int b = (m_colors[i].blue - icolors[i].blue) >> 8;
-                            pixel = (r * r) + (g * g) + (b * b);
-
-                            if (pixel < chk) {
-                                chk = pixel;
-                                close = ii;
-                            }
-
-                            m_colors[i].red = icolors[close].red;
-                            m_colors[i].green = icolors[close].green;
-                            m_colors[i].blue = icolors[close].blue;
-
-                            if (XAllocColor(disp, m_colormap, &m_colors[i])) {
-                                m_colors[i].flags = DoRed|DoGreen|DoBlue;
-                                break;
-                            }
-                        }
-                    }
-                }
             }
 
+            allocateUnallocatedColors(m_colors, disp, m_colormap, m_screen_depth);
             break;
         }
 
     default:
         _FB_USES_NLS;
-        cerr<<"FbTk::ImageControl: "<<_FBTKTEXT(Error, UnsupportedVisual, "Unsupported visual", "A visual is a technical term in X")<<endl;
+        cerr<<"FbTk::ImageControl: "<<_FBTK_CONSOLETEXT(Error, UnsupportedVisual, "Unsupported visual", "A visual is a technical term in X")<<endl;
         break;
     }
 }
 
-}; // end namespace FbTk
+} // end namespace FbTk
