@@ -51,7 +51,6 @@
 #include "FbTk/BoolMenuItem.hh"
 #include "FbTk/IntMenuItem.hh"
 #include "FbTk/Shape.hh"
-#include "FbTk/SimpleObserver.hh"
 #include "FbTk/MemFun.hh"
 #include "FbTk/STLUtil.hh"
 #include "FbTk/Util.hh"
@@ -101,8 +100,8 @@ const ToolbarPlacementString placement_strings[] = {
     { Toolbar::LEFTBOTTOM, "LeftBottom" },
     { Toolbar::LEFTCENTER, "LeftCenter" },
     { Toolbar::LEFTTOP, "LeftTop" },
-    { Toolbar::RIGHTCENTER, "RightCenter" },
     { Toolbar::RIGHTBOTTOM, "RightBottom" },
+    { Toolbar::RIGHTCENTER, "RightCenter" },
     { Toolbar::RIGHTTOP, "RightTop" }
 };
 
@@ -115,8 +114,8 @@ string FbTk::Resource<Toolbar::Placement>::
 getString() const {
 
     size_t i = (m_value == FbTk::Util::clamp(m_value, Toolbar::TOPLEFT, Toolbar::RIGHTTOP)
-                ? m_value 
-                : Toolbar::DEFAULT) - 1;
+                ? m_value
+                : Toolbar::DEFAULT) - Toolbar::TOPLEFT;
     return placement_strings[i].str;
 }
 
@@ -190,15 +189,15 @@ Toolbar::Toolbar(BScreen &scrn, FbTk::Layer &layer, size_t width):
     m_layeritem(frame.window, layer),
     m_layermenu(scrn.menuTheme(),
                 scrn.imageControl(),
-                *scrn.layerManager().getLayer(Layer::MENU),
+                *scrn.layerManager().getLayer(ResourceLayer::MENU),
                 this,
                 true),
     m_placementmenu(scrn.menuTheme(),
                     scrn.imageControl(),
-                    *scrn.layerManager().getLayer(Layer::MENU)),
+                    *scrn.layerManager().getLayer(ResourceLayer::MENU)),
     m_toolbarmenu(scrn.menuTheme(),
                   scrn.imageControl(),
-                  *scrn.layerManager().getLayer(Layer::MENU)),
+                  *scrn.layerManager().getLayer(ResourceLayer::MENU)),
 #ifdef XINERAMA
     m_xineramaheadmenu(0),
 #endif // XINERAMA
@@ -215,7 +214,7 @@ Toolbar::Toolbar(BScreen &scrn, FbTk::Layer &layer, size_t width):
                        scrn.name() + ".toolbar.widthPercent", scrn.altName() + ".Toolbar.WidthPercent"),
     m_rc_alpha(scrn.resourceManager(), 255,
                        scrn.name() + ".toolbar.alpha", scrn.altName() + ".Toolbar.Alpha"),
-    m_rc_layernum(scrn.resourceManager(), Layer(Layer::DOCK),
+    m_rc_layernum(scrn.resourceManager(), ResourceLayer(ResourceLayer::DOCK),
                   scrn.name() + ".toolbar.layer", scrn.altName() + ".Toolbar.Layer"),
     m_rc_on_head(scrn.resourceManager(), 1,
                  scrn.name() + ".toolbar.onhead", scrn.altName() + ".Toolbar.onHead"),
@@ -227,12 +226,13 @@ Toolbar::Toolbar(BScreen &scrn, FbTk::Layer &layer, size_t width):
     m_shape(new FbTk::Shape(frame.window, 0)),
     m_resize_lock(false) {
     _FB_USES_NLS;
-    // NOTE: first subject is always the rearrangeItem !
-    m_observers.push_back(makeObserver(*this, &Toolbar::rearrangeItems));
+
+    // get this on antialias change
+    m_signal_tracker.join(screen().reconfigureSig(),
+            FbTk::MemFunIgnoreArgs(*this, &Toolbar::reconfigure));
+
     // we need to get notified when the theme is reloaded
-    m_observers.push_back(makeObserver(*this, &Toolbar::reconfigure));
-    m_theme.reconfigSig().attach(m_observers.back());
-    screen().reconfigureSig().attach(m_observers.back()); // get this on antialias change
+    m_signal_tracker.join(m_theme.reconfigSig(), FbTk::MemFun(*this, &Toolbar::reconfigure));
 
     // listen to screen size changes
     m_signal_tracker.join(screen().resizeSig(),
@@ -422,9 +422,8 @@ void Toolbar::reconfigure() {
                 if (item == 0)
                     continue;
                 m_item_list.push_back(item);
-                // attach to first observer ( which must be rearrangeItems )
-                item->resizeSig().attach(m_observers[0]);
-
+                m_signal_tracker.join(item->resizeSig(),
+                        FbTk::MemFun(*this, &Toolbar::rearrangeItems));
             }
             // show all items
             frame.window.showSubwindows();
@@ -829,7 +828,7 @@ void Toolbar::setupMenus(bool skip_new_placement) {
                       new XineramaHeadMenu<Toolbar>(screen().menuTheme(),
                                                     screen(),
                                                     screen().imageControl(),
-                                                    *screen().layerManager().getLayer(::Layer::MENU),
+                                                    *screen().layerManager().getLayer(::ResourceLayer::MENU),
                                                     *this,
                                                     _FB_XTEXT(Toolbar, OnHead, "Toolbar on Head",
                                                               "Title of toolbar on head menu")));
