@@ -20,11 +20,9 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: ToolbarItem.cc 4199 2006-02-16 06:53:05Z mathias $
-
 #include "ToolbarItem.hh"
 
-ToolbarItem::ToolbarItem(Type type):m_type(type) {
+ToolbarItem::ToolbarItem(Type type):m_type(type), m_orientation(FbTk::ROT0) {
 
 }
 
